@@ -7,7 +7,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2024 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2025 Gerardo Orellana <hello @ goaccess.io>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -54,6 +54,7 @@ static const GEnum CSTM_COLORS[] = {
   {"COLOR_MTRC_VISITORS"          , COLOR_MTRC_VISITORS},
   {"COLOR_MTRC_HITS_PERC"         , COLOR_MTRC_HITS_PERC},
   {"COLOR_MTRC_VISITORS_PERC"     , COLOR_MTRC_VISITORS_PERC},
+  {"COLOR_MTRC_BW_PERC"           , COLOR_MTRC_BW_PERC},
   {"COLOR_MTRC_BW"                , COLOR_MTRC_BW},
   {"COLOR_MTRC_AVGTS"             , COLOR_MTRC_AVGTS},
   {"COLOR_MTRC_CUMTS"             , COLOR_MTRC_CUMTS},
@@ -63,8 +64,12 @@ static const GEnum CSTM_COLORS[] = {
   {"COLOR_MTRC_DATA"              , COLOR_MTRC_DATA},
   {"COLOR_MTRC_HITS_PERC_MAX"     , COLOR_MTRC_HITS_PERC_MAX},
   {"COLOR_MTRC_VISITORS_PERC_MAX" , COLOR_MTRC_VISITORS_PERC_MAX},
+  {"COLOR_MTRC_BW_PERC_MAX"       , COLOR_MTRC_BW_PERC_MAX},
   {"COLOR_PANEL_COLS"             , COLOR_PANEL_COLS},
   {"COLOR_BARS"                   , COLOR_BARS},
+  {"COLOR_SUBBARS"                , COLOR_SUBBARS},
+  {"COLOR_CHART_AXIS"             , COLOR_CHART_AXIS},
+  {"COLOR_CHART_VALUES"           , COLOR_CHART_VALUES},
   {"COLOR_ERROR"                  , COLOR_ERROR},
   {"COLOR_SELECTED"               , COLOR_SELECTED},
   {"COLOR_PANEL_ACTIVE"           , COLOR_PANEL_ACTIVE},
@@ -101,8 +106,13 @@ static const char *const colors256_mono[] = {
   "COLOR_MTRC_HITS_PERC_MAX     color3:color-1 bold VISIT_TIMES",
   "COLOR_MTRC_VISITORS_PERC     color0:color-1 bold",
   "COLOR_MTRC_VISITORS_PERC_MAX color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC           color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC_MAX       color0:color-1 bold",
   "COLOR_PANEL_COLS             color7:color-1",
   "COLOR_BARS                   color7:color-1",
+  "COLOR_SUBBARS                color8:color-1",
+  "COLOR_CHART_AXIS             color7:color-1",
+  "COLOR_CHART_VALUES           color7:color-1",
   "COLOR_ERROR                  color7:color1",
   "COLOR_SELECTED               color7:color8",
   "COLOR_PANEL_ACTIVE           color0:color3",
@@ -139,8 +149,13 @@ static const char *const colors256_green[] = {
   "COLOR_MTRC_HITS_PERC_MAX     color3:color-1 bold VISIT_TIMES",
   "COLOR_MTRC_VISITORS_PERC     color0:color-1 bold",
   "COLOR_MTRC_VISITORS_PERC_MAX color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC           color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC_MAX       color0:color-1 bold",
   "COLOR_PANEL_COLS             color7:color-1",
   "COLOR_BARS                   color7:color-1",
+  "COLOR_SUBBARS                color8:color-1",
+  "COLOR_CHART_AXIS             color7:color-1",
+  "COLOR_CHART_VALUES           color7:color-1",
   "COLOR_ERROR                  color7:color1",
   "COLOR_SELECTED               color7:color8",
   "COLOR_PANEL_ACTIVE           color0:color3",
@@ -177,8 +192,13 @@ static const char *const colors256_monokai[] = {
   "COLOR_MTRC_HITS_PERC_MAX     color208:color-1 VISIT_TIMES",
   "COLOR_MTRC_VISITORS_PERC     color187:color-1",
   "COLOR_MTRC_VISITORS_PERC_MAX color208:color-1",
+  "COLOR_MTRC_BW_PERC           color187:color-1",
+  "COLOR_MTRC_BW_PERC_MAX       color208:color-1",
   "COLOR_PANEL_COLS             color242:color-1",
-  "COLOR_BARS                   color186:color-1",
+  "COLOR_BARS                   color240:color-1",
+  "COLOR_SUBBARS                color250:color-1",
+  "COLOR_CHART_AXIS             color241:color-1",
+  "COLOR_CHART_VALUES           color248:color-1",
   "COLOR_ERROR                  color231:color197",
   "COLOR_SELECTED               color0:color215",
   "COLOR_PANEL_ACTIVE           color7:color240",
@@ -215,8 +235,13 @@ static const char *const colors8_mono[] = {
   "COLOR_MTRC_HITS_PERC_MAX     color3:color-1 bold VISIT_TIMES",
   "COLOR_MTRC_VISITORS_PERC     color0:color-1 bold",
   "COLOR_MTRC_VISITORS_PERC_MAX color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC           color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC_MAX       color0:color-1 bold",
   "COLOR_PANEL_COLS             color7:color-1",
   "COLOR_BARS                   color7:color-1",
+  "COLOR_SUBBARS                color8:color-1",
+  "COLOR_CHART_AXIS             color7:color-1",
+  "COLOR_CHART_VALUES           color7:color-1",
   "COLOR_ERROR                  color7:color1",
   "COLOR_SELECTED               color0:color7",
   "COLOR_PANEL_ACTIVE           color0:color3",
@@ -253,8 +278,13 @@ static const char *const colors8_green[] = {
   "COLOR_MTRC_HITS_PERC_MAX     color3:color-1 bold VISIT_TIMES",
   "COLOR_MTRC_VISITORS_PERC     color0:color-1 bold",
   "COLOR_MTRC_VISITORS_PERC_MAX color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC           color0:color-1 bold",
+  "COLOR_MTRC_BW_PERC_MAX       color0:color-1 bold",
   "COLOR_PANEL_COLS             color7:color-1",
   "COLOR_BARS                   color2:color-1",
+  "COLOR_SUBBARS                color8:color-1",
+  "COLOR_CHART_AXIS             color2:color-1",
+  "COLOR_CHART_VALUES           color2:color-1",
   "COLOR_ERROR                  color7:color1",
   "COLOR_SELECTED               color0:color7",
   "COLOR_PANEL_ACTIVE           color0:color3",
@@ -283,8 +313,13 @@ static const char *const nocolors[] = {
   "COLOR_MTRC_HITS_PERC_MAX     color0:color-1",
   "COLOR_MTRC_VISITORS_PERC     color0:color-1",
   "COLOR_MTRC_VISITORS_PERC_MAX color0:color-1",
+  "COLOR_MTRC_BW_PERC           color0:color-1",
+  "COLOR_MTRC_BW_PERC_MAX       color0:color-1",
   "COLOR_PANEL_COLS             color0:color-1",
   "COLOR_BARS                   color0:color-1",
+  "COLOR_SUBBARS                color0:color-1",
+  "COLOR_CHART_AXIS             color0:color-1",
+  "COLOR_CHART_VALUES           color0:color-1",
   "COLOR_ERROR                  color0:color-1",
   "COLOR_SELECTED               color0:color-1 reverse",
   "COLOR_PANEL_ACTIVE           color0:color-1 reverse",
