@@ -7,7 +7,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2024 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2025 Gerardo Orellana <hello @ goaccess.io>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -79,25 +79,26 @@ static const GEnum enum_modules[] = {
 };
 /* *INDENT-ON* */
 
-/* Get number of items per panel to parse.
+/* Get number of items per panel/level to parse.
  *
- * The number of items per panel is returned. */
-int
-get_max_choices (void) {
+ * If is_sub is true, returns sub-item limits, otherwise returns panel limits.
+ * The number of items per panel/level is returned. */
+static uint32_t
+get_max_choices_internal (int is_sub) {
   char *csv = NULL, *json = NULL, *html = NULL;
   int max = MAX_CHOICES;
+  int rt_default = is_sub ? MAX_CHOICES_SUB_RT : MAX_CHOICES_RT;
 
   /* no max choices, return defaults */
   if (conf.max_items <= 0)
-    return conf.real_time_html ? MAX_CHOICES_RT : (conf.date_spec_hr ==
-                                                   2 ? MAX_CHOICES_MINUTE : MAX_CHOICES);
+    return conf.real_time_html ? rt_default : (conf.date_spec_hr == 2 ? MAX_CHOICES_MINUTE : MAX_CHOICES);
 
   /* TERM */
   if (!conf.output_stdout)
     return conf.max_items > MAX_CHOICES ? MAX_CHOICES : conf.max_items;
 
   /* REAL-TIME STDOUT */
-  /* real time HTML, display max rt choices */
+  /* real time HTML, display max rt choices - always cap at MAX_CHOICES_RT */
   if (conf.real_time_html)
     return conf.max_items > MAX_CHOICES_RT ? MAX_CHOICES_RT : conf.max_items;
 
@@ -121,6 +122,18 @@ get_max_choices (void) {
   return max;
 }
 
+/* Get number of items per panel to parse. */
+uint32_t
+get_max_choices (void) {
+  return get_max_choices_internal (0);
+}
+
+/* Get number of sub-items per hierarchy level to parse. */
+uint32_t
+get_max_choices_sub (void) {
+  return get_max_choices_internal (1);
+}
+
 /* Calculate a percentage.
  *
  * The percentage is returned. */
