@@ -6,7 +6,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2024 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2025 Gerardo Orellana <hello @ goaccess.io>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,7 +43,7 @@
 #define __attribute__(x) /**/
 #endif
 #define GO_UNUSED __attribute__((unused))
-#define GO_VERSION 		"1.9.3"
+#define GO_VERSION 		"1.10"
 #define GO_WEBSITE 		"https://goaccess.io/"
 extern struct tm now_tm;
 
@@ -61,6 +61,8 @@ extern struct tm now_tm;
 #define MAX_CHOICES          366
 /* real-time */
 #define MAX_CHOICES_RT        50
+/* real-time sub-items */
+#define MAX_CHOICES_SUB_RT    10
 /* max default items when date-spec = min */
 #define MAX_CHOICES_MINUTE  1440 /* 24hrs */
 
@@ -164,13 +166,7 @@ typedef struct GMetrics {
 
   uint64_t hits;
   uint64_t visitors;
-
-  /* holder has a numeric value, while
-   * dashboard has a displayable string value */
-  union {
-    char *sbw;
-    uint64_t nbw;
-  } bw;
+  uint64_t nbw;
 
   /* holder has a numeric value, while
    * dashboard has a displayable string value */
@@ -194,20 +190,24 @@ typedef struct GMetrics {
   } maxts;
 } GMetrics;
 
+/* Forward declaration for recursive sub-items */
+typedef struct GSubList_ GSubList;
+
 /* Holder sub item */
 typedef struct GSubItem_ {
   GModule module;
   GMetrics *metrics;
+  GSubList *sub_list;
   struct GSubItem_ *prev;
   struct GSubItem_ *next;
 } GSubItem;
 
 /* Double linked-list of sub items */
-typedef struct GSubList_ {
-  int size;
+struct GSubList_ {
+  uint32_t size;
   struct GSubItem_ *head;
   struct GSubItem_ *tail;
-} GSubList;
+};
 
 /* Holder item */
 typedef struct GHolderItem_ {
@@ -219,10 +219,12 @@ typedef struct GHolderItem_ {
 typedef struct GHolder_ {
   GHolderItem *items;           /* holder items */
   GModule module;               /* current module  */
-  int idx;                      /* holder index  */
-  int holder_size;              /* number of allocated items */
+  uint32_t idx;                      /* holder index  */
+  uint32_t holder_size;              /* number of allocated items */
   uint32_t ht_size;             /* size of the hash table/store */
-  int sub_items_size;           /* number of sub items  */
+  uint32_t sub_items_size;           /* number of sub items  */
+  uint32_t max_choices;              /* max items at root level */
+  uint32_t max_choices_sub;          /* max items at sub-item levels */
 } GHolder;
 
 /* Enum-to-string */
@@ -265,7 +267,8 @@ void free_agents_array (GAgents *agents);
 const char *enum2str (const GEnum map[], int len, int idx);
 const char *get_module_str (GModule module);
 float get_percentage (unsigned long long total, unsigned long long hit);
-int get_max_choices (void);
+uint32_t get_max_choices (void);
+uint32_t get_max_choices_sub (void);
 int get_module_enum (const char *str);
 int has_timestamp (const char *fmt);
 int str2enum (const GEnum map[], int len, const char *str);
