@@ -84,6 +84,7 @@
 #define MTRC_VISITORS_SHORT_LBL  _( "Vis.")
 #define MTRC_VISITORS_PERC_LBL   _( "v%")
 #define MTRC_BW_LBL              _( "Tx. Amount")
+#define MTRC_BW_PERC_LBL         _( "tx%")
 #define MTRC_AVGTS_LBL           _( "Avg. T.S.")
 #define MTRC_CUMTS_LBL           _( "Cum. T.S.")
 #define MTRC_MAXTS_LBL           _( "Max. T.S.")
@@ -290,6 +291,8 @@
 /* Common UI Errors */
 #define ERR_FORMAT_HEADER              \
   _("Format Errors - Verify your log/date/time format")
+#define HINT_INVALID_REQUESTS          \
+  _("Use --invalid-requests option to store such lines in a file.")
 #define ERR_FORMAT_NO_DATE_FMT         \
   _("No date format was found on your conf file.")
 #define ERR_FORMAT_NO_LOG_FMT          \
