@@ -7,7 +7,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2024 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2025 Gerardo Orellana <hello @ goaccess.io>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -224,8 +224,8 @@ cmp_bw_desc (const void *a, const void *b) {
   const GHolderItem *ia = a;
   const GHolderItem *ib = b;
 
-  uint64_t va = ia->metrics->bw.nbw;
-  uint64_t vb = ib->metrics->bw.nbw;
+  uint64_t va = ia->metrics->nbw;
+  uint64_t vb = ib->metrics->nbw;
 
   return (va < vb) - (va > vb);
 }
@@ -236,8 +236,8 @@ cmp_bw_asc (const void *a, const void *b) {
   const GHolderItem *ia = a;
   const GHolderItem *ib = b;
 
-  uint64_t va = ia->metrics->bw.nbw;
-  uint64_t vb = ib->metrics->bw.nbw;
+  uint64_t va = ia->metrics->nbw;
+  uint64_t vb = ib->metrics->nbw;
 
   return (va > vb) - (va < vb);
 }
