@@ -7,7 +7,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2024 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2025 Gerardo Orellana <hello @ goaccess.io>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -47,6 +47,9 @@
 #include <inttypes.h>
 #include <regex.h>
 #include <pthread.h>
+#include <unistd.h>
+#include <libgen.h>
+#include <limits.h>
 
 #include <netinet/in.h>
 #include <sys/socket.h>
@@ -104,6 +107,12 @@ static const char *const codes[600] = {
   [598] = STATUS_CODE_598, STATUS_CODE_599,
 };
 
+const char *
+basename_only (const char *path) {
+  const char *base = strrchr (path, '/');
+  return base ? base + 1 : path;
+}
+
 /* Return part of a string
  *
  * On error NULL is returned.
@@ -970,6 +979,29 @@ u642str (uint64_t d, int width) {
   return s;
 }
 
+/* Decodes the given URL-encoded string.
+ *
+ * On success, the decoded string is assigned to the output buffer. */
+#define B16TOD(x) (((x) >= '0' && (x) <= '9') ? ((x) - '0') : (toupper((unsigned char) (x)) - 'A' + 10))
+void
+decode_hex (char *url, char *out, int decode_plus) {
+  char *ptr;
+  const char *c;
+  for (c = url, ptr = out; *c; c++) {
+    if (*c != '%' || !isxdigit ((unsigned char) c[1]) || !isxdigit ((unsigned char) c[2])) {
+      if (decode_plus && *c == '+') {
+        *ptr++ = ' ';
+      } else {
+        *ptr++ = *c;
+      }
+    } else {
+      *ptr++ = (char) ((B16TOD (c[1]) * 16) + (B16TOD (c[2])));
+      c += 2;
+    }
+  }
+  *ptr = 0;
+}
+
 /* Convert the given float to a string with the ability to add some
  * padding.
  *
@@ -1221,3 +1253,52 @@ out:
 
   return dest;
 }
+
+int
+is_writable_path (const char *path) {
+  char *copy = NULL, *dir_path = NULL;
+  char dir_path_copy[PATH_MAX] = { 0 };
+  int result = 0;
+
+  if (path == NULL) {
+    fprintf (stderr, "Path is NULL\n");
+    return 0;
+  }
+
+  /* Make a copy of the path because dirname might modify it */
+  copy = strdup (path);
+  if (copy == NULL) {
+    fprintf (stderr, "Memory allocation failed\n");
+    return 0;
+  }
+
+  /* Get the directory part of the path */
+  dir_path = dirname (copy);
+  snprintf (dir_path_copy, sizeof (dir_path_copy), "%s", dir_path);
+
+  /* Check if the directory is writable */
+  result = access (dir_path, W_OK);
+  free (copy);
+
+  if (result == 0) {
+    /* Directory exists and is writable */
+    return 1;
+  }
+
+  switch (errno) {
+  case ENOENT:
+    fprintf (stderr, "Directory does not exist: %s\n", dir_path_copy);
+    break;
+  case EACCES:
+    fprintf (stderr, "No write permission for directory: %s\n", dir_path_copy);
+    break;
+  case EROFS:
+    fprintf (stderr, "Directory is on a read-only file system: %s\n", dir_path_copy);
+    break;
+  default:
+    fprintf (stderr, "Unknown error (errno %d) for directory: %s\n", errno, dir_path_copy);
+    break;
+  }
+
+  return 0;
+}
